@@ -1,23 +1,23 @@
-# encoding: utf-8
+# -*- coding: utf-8 -*-
 """Module providing 'sequence awareness'."""
 # std imports
-import functools
-import textwrap
-import math
 import re
+import math
+import textwrap
+
+# 3rd party
+import six
+from wcwidth import wcwidth
 
 # local
 from blessed._capabilities import CAPABILITIES_CAUSE_MOVEMENT
 
-# 3rd party
-import wcwidth
-import six
-
 __all__ = ('Sequence', 'SequenceTextWrapper', 'iter_parse', 'measure_length')
 
 
 class Termcap(object):
     """Terminal capability of given variable name and pattern."""
+
     def __init__(self, name, pattern, attribute):
         """
         Class initializer.
@@ -38,11 +38,13 @@ class Termcap(object):
 
     @property
     def named_pattern(self):
+        """Regular expression pattern for capability with named group."""
         # pylint: disable=redundant-keyword-arg
         return '(?P<{self.name}>{self.pattern})'.format(self=self)
 
     @property
     def re_compiled(self):
+        """Compiled regular expression pattern for capability."""
         if self._re_compiled is None:
             self._re_compiled = re.compile(self.pattern)
         return self._re_compiled
@@ -96,9 +98,10 @@ class Termcap(object):
             capability to build for. When ``nparams`` is non-zero, it
             must be a callable unicode string (such as the result from
             ``getattr(term, 'bold')``.
-        :arg attribute: The terminfo(5) capability name by which this
+        :arg str attribute: The terminfo(5) capability name by which this
             pattern is known.
         :arg int nparams: number of positional arguments for callable.
+        :arg int numeric: Value to substitute into capability to when generating pattern
         :arg bool match_grouped: If the numeric pattern should be
             grouped, ``(\d+)`` when ``True``, ``\d+`` default.
         :arg bool match_any: When keyword argument ``nparams`` is given,
@@ -108,6 +111,8 @@ class Termcap(object):
             pattern ``(\d+)`` in builder.
         :arg bool match_optional: When ``True``, building of numeric patterns
             containing ``(\d+)`` will be built as optional, ``(\d+)?``.
+        :rtype: blessed.sequences.Termcap
+        :returns: Terminal capability instance for given capability definition
         """
         _numeric_regex = r'\d+'
         if match_grouped:
@@ -151,11 +156,14 @@ class SequenceTextWrapper(textwrap.TextWrapper):
         """
         Sequence-aware variant of :meth:`textwrap.TextWrapper._wrap_chunks`.
 
-        This simply ensures that word boundaries are not broken mid-sequence,
-        as standard python textwrap would incorrectly determine the length
-        of a string containing sequences, and may also break consider sequences
-        part of a "word" that may be broken by hyphen (``-``), where this
-        implementation corrects both.
+        :raises ValueError: ``self.width`` is not a positive integer
+        :rtype: list
+        :returns: text chunks adjusted for width
+
+        This simply ensures that word boundaries are not broken mid-sequence, as standard python
+        textwrap would incorrectly determine the length of a string containing sequences, and may
+        also break consider sequences part of a "word" that may be broken by hyphen (``-``), where
+        this implementation corrects both.
         """
         lines = []
         if self.width <= 0 or not isinstance(self.width, int):
@@ -170,21 +178,17 @@ class SequenceTextWrapper(textwrap.TextWrapper):
         while chunks:
             cur_line = []
             cur_len = 0
-            if lines:
-                indent = self.subsequent_indent
-            else:
-                indent = self.initial_indent
+            indent = self.subsequent_indent if lines else self.initial_indent
             width = self.width - len(indent)
             if drop_whitespace and (
                     Sequence(chunks[-1], term).strip() == '' and lines):
                 del chunks[-1]
             while chunks:
                 chunk_len = Sequence(chunks[-1], term).length()
-                if cur_len + chunk_len <= width:
-                    cur_line.append(chunks.pop())
-                    cur_len += chunk_len
-                else:
+                if cur_len + chunk_len > width:
                     break
+                cur_line.append(chunks.pop())
+                cur_len += chunk_len
             if chunks and Sequence(chunks[-1], term).length() > width:
                 self._handle_long_word(chunks, cur_line, cur_len, width)
             if drop_whitespace and (
@@ -198,19 +202,14 @@ class SequenceTextWrapper(textwrap.TextWrapper):
         """
         Sequence-aware :meth:`textwrap.TextWrapper._handle_long_word`.
 
-        This simply ensures that word boundaries are not broken mid-sequence,
-        as standard python textwrap would incorrectly determine the length
-        of a string containing sequences, and may also break consider sequences
-        part of a "word" that may be broken by hyphen (``-``), where this
-        implementation corrects both.
+        This simply ensures that word boundaries are not broken mid-sequence, as standard python
+        textwrap would incorrectly determine the length of a string containing sequences, and may
+        also break consider sequences part of a "word" that may be broken by hyphen (``-``), where
+        this implementation corrects both.
         """
         # Figure out when indent is larger than the specified width, and make
         # sure at least one character is stripped off on every pass
-        if width < 1:
-            space_left = 1
-        else:
-            space_left = width - cur_len
-
+        space_left = 1 if width < 1 else width - cur_len
         # If we're allowed to break long words, then do so: put as much
         # of the next chunk onto the current line as will fit.
 
@@ -252,10 +251,11 @@ class Sequence(six.text_type):
     """
 
     def __new__(cls, sequence_text, term):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor.
 
-        :arg sequence_text: A string that may contain sequences.
+        :arg str sequence_text: A string that may contain sequences.
         :arg blessed.Terminal term: :class:`~.Terminal` instance.
         """
         new = six.text_type.__new__(cls, sequence_text)
@@ -266,10 +266,10 @@ class Sequence(six.text_type):
         """
         Return string containing sequences, left-adjusted.
 
-        :arg int width: Total width given to right-adjust ``text``.  If
+        :arg int width: Total width given to left-adjust ``text``.  If
             unspecified, the width of the attached terminal is used (default).
         :arg str fillchar: String for padding right-of ``text``.
-        :returns: String of ``text``, right-aligned by ``width``.
+        :returns: String of ``text``, left-aligned by ``width``.
         :rtype: str
         """
         rightside = fillchar * int(
@@ -307,6 +307,34 @@ class Sequence(six.text_type):
             (max(0.0, math.ceil(split))) / float(len(fillchar)))
         return u''.join((leftside, self, rightside))
 
+    def truncate(self, width):
+        """
+        Truncate a string in a sequence-aware manner.
+
+        Any printable characters beyond ``width`` are removed, while all
+        sequences remain in place. Horizontal Sequences are first expanded
+        by :meth:`padd`.
+
+        :arg int width: The printable width to truncate the string to.
+        :rtype: str
+        :returns: String truncated to at most ``width`` printable characters.
+        """
+        output = ""
+        current_width = 0
+        parsed_seq = iter_parse(self._term, self.padd())
+
+        # Retain all text until non-cap width reaches desired width
+        for text, cap in parsed_seq:
+            if not cap:
+                # use wcwidth clipped to 0 because it can sometimes return -1
+                current_width += max(wcwidth(text), 0)
+                if current_width > width:
+                    break
+            output += text
+
+        # Return with remaining caps appended
+        return output + ''.join(text for text, cap in parsed_seq if cap)
+
     def length(self):
         r"""
         Return the printable length of string containing sequences.
@@ -319,37 +347,30 @@ class Sequence(six.text_type):
         Some characters may consume more than one cell, mainly those CJK
         Unified Ideographs (Chinese, Japanese, Korean) defined by Unicode
         as half or full-width characters.
+
+        For example:
+
+            >>> from blessed import Terminal
+            >>> from blessed.sequences import Sequence
+            >>> term = Terminal()
+            >>> msg = term.clear + term.red(u'コンニチハ')
+            >>> Sequence(msg, term).length()
+            10
+
+        .. note:: Although accounted for, strings containing sequences such
+            as ``term.clear`` will not give accurate returns, it is not
+            considered lengthy (a length of 0).
         """
         # because control characters may return -1, "clip" their length to 0.
-        clip = functools.partial(max, 0)
-        return sum(clip(wcwidth.wcwidth(w_char))
-                   for w_char in self.strip_seqs())
-
-    # we require ur"" for the docstring, but it is not supported by all
-    # python versions.
-    if length.__doc__ is not None:
-        length.__doc__ += (
-            u"""
-            For example:
-
-                >>> from blessed import Terminal
-                >>> from blessed.sequences import Sequence
-                >>> term = Terminal()
-                >>> msg = term.clear + term.red(u'コンニチハ'), term
-                >>> Sequence(msg).length()
-                10
-
-            .. note:: Although accounted for, strings containing sequences such
-               as ``term.clear`` will not give accurate returns, it is not
-               considered lengthy (a length of 0).
-            """)
+        return sum(max(wcwidth(w_char), 0) for w_char in self.padd(strip=True))
 
     def strip(self, chars=None):
         """
-        Return string of sequences, leading, and trailing whitespace removed.
+        Return string of sequences, leading and trailing whitespace removed.
 
         :arg str chars: Remove characters in chars instead of whitespace.
         :rtype: str
+        :returns: string of sequences with leading and trailing whitespace removed.
         """
         return self.strip_seqs().strip(chars)
 
@@ -359,6 +380,7 @@ class Sequence(six.text_type):
 
         :arg str chars: Remove characters in chars instead of whitespace.
         :rtype: str
+        :returns: string of sequences with leading removed.
         """
         return self.strip_seqs().lstrip(chars)
 
@@ -368,6 +390,7 @@ class Sequence(six.text_type):
 
         :arg str chars: Remove characters in chars instead of whitespace.
         :rtype: str
+        :returns: string of sequences with trailing removed.
         """
         return self.strip_seqs().rstrip(chars)
 
@@ -376,15 +399,17 @@ class Sequence(six.text_type):
         Return ``text`` stripped of only its terminal sequences.
 
         :rtype: str
+        :returns: Text with terminal sequences removed
         """
-        gen = iter_parse(self._term, self.padd())
-        return u''.join(text for text, cap in gen if not cap)
+        return self.padd(strip=True)
 
-    def padd(self):
+    def padd(self, strip=False):
         """
         Return non-destructive horizontal movement as destructive spacing.
 
+        :arg bool strip: Strip terminal sequences
         :rtype: str
+        :returns: Text adjusted for horizontal movement
         """
         outp = ''
         for text, cap in iter_parse(self._term, self):
@@ -397,7 +422,7 @@ class Sequence(six.text_type):
                 outp += ' ' * value
             elif value < 0:
                 outp = outp[:value]
-            else:
+            elif not strip:
                 outp += text
         return outp
 
@@ -410,7 +435,7 @@ def iter_parse(term, text):
     :class:`str` of length 1.  Otherwise, ``text`` is a full
     matching sequence of given capability.
     """
-    for match in re.finditer(term._caps_compiled_any, text):
+    for match in term._caps_compiled_any.finditer(text):  # pylint: disable=protected-access
         name = match.lastgroup
         value = match.group(name)
         if name == 'MISMATCH':
@@ -424,6 +449,7 @@ def measure_length(text, term):
     .. deprecated:: 1.12.0.
 
     :rtype: int
+    :returns: Length of the first sequence in the string
     """
     try:
         text, capability = next(iter_parse(term, text))
