@@ -37,6 +37,8 @@ from .formatters import (COLORS,
                          resolve_capability)
 from ._capabilities import CAPABILITY_DATABASE, CAPABILITIES_ADDITIVES, CAPABILITIES_RAW_MIXIN
 
+# isort: off
+
 try:
     InterruptedError
 except NameError:
@@ -44,22 +46,19 @@ except NameError:
     # pylint: disable=redefined-builtin
     InterruptedError = select.error
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    # python 2.6 requires 3rd party library (backport)
-    from ordereddict import OrderedDict  # pylint: disable=import-error
-
 
 HAS_TTY = True
 if platform.system() == 'Windows':
+    IS_WINDOWS = True
     import jinxed as curses  # pylint: disable=import-error
+    from jinxed.win32 import get_console_input_encoding  # pylint: disable=import-error
 else:
+    IS_WINDOWS = False
     import curses
 
     try:
-        import termios
         import fcntl
+        import termios
         import tty
     except ImportError:
         _TTY_METHODS = ('setraw', 'cbreak', 'kbhit', 'height', 'width')
@@ -167,19 +166,16 @@ class Terminal(object):
         self._is_a_tty = False
         self.__init__streams()
 
-        if platform.system() == 'Windows' and self._init_descriptor is not None:
+        if IS_WINDOWS and self._init_descriptor is not None:
             self._kind = kind or curses.get_term(self._init_descriptor)
         else:
             self._kind = kind or os.environ.get('TERM', 'dumb') or 'dumb'
 
         self._does_styling = False
-        if force_styling:
+        if force_styling is None and self.is_a_tty:
+            self.errors.append('force_styling is None')
+        elif force_styling or self.is_a_tty:
             self._does_styling = True
-        elif self.is_a_tty:
-            if force_styling is None:
-                self.errors.append('force_styling is None')
-            else:
-                self._does_styling = True
 
         if self.does_styling:
             # Initialize curses (call setupterm), so things like tigetstr() work.
@@ -240,7 +236,7 @@ class Terminal(object):
         if self._stream in (sys.__stdout__, sys.__stderr__):
             try:
                 self._keyboard_fd = sys.__stdin__.fileno()
-            except ValueError as err:
+            except (AttributeError, ValueError) as err:
                 self.errors.append('Unable to determine input stream file descriptor: %s' % err)
             else:
                 # _keyboard_fd only non-None if both stdin and stdout is a tty.
@@ -265,8 +261,7 @@ class Terminal(object):
         self._color_distance_algorithm = 'cie2000'
         if not self.does_styling:
             self.number_of_colors = 0
-        elif platform.system() == 'Windows' or (
-                os.environ.get('COLORTERM') in ('truecolor', '24bit')):
+        elif IS_WINDOWS or os.environ.get('COLORTERM') in ('truecolor', '24bit'):
             self.number_of_colors = 1 << 24
         else:
             self.number_of_colors = max(0, curses.tigetnum('colors') or -1)
@@ -278,7 +273,7 @@ class Terminal(object):
     def __init__capabilities(self):
         # important that we lay these in their ordered direction, so that our
         # preferred, 'color' over 'set_a_attributes1', for example.
-        self.caps = OrderedDict()
+        self.caps = collections.OrderedDict()
 
         # some static injected patterns, esp. without named attribute access.
         for name, (attribute, pattern) in CAPABILITIES_ADDITIVES.items():
@@ -331,7 +326,13 @@ class Terminal(object):
 
         if self._keyboard_fd is not None:
             # set input encoding and initialize incremental decoder
-            self._encoding = locale.getpreferredencoding() or 'UTF-8'
+
+            if IS_WINDOWS:
+                self._encoding = get_console_input_encoding() \
+                    or locale.getpreferredencoding() or 'UTF-8'
+            else:
+                self._encoding = locale.getpreferredencoding() or 'UTF-8'
+
             try:
                 self._keyboard_decoder = codecs.getincrementaldecoder(self._encoding)()
             except LookupError as err:
@@ -567,8 +568,9 @@ class Terminal(object):
             always ensure the return value is checked for ``(-1, -1)``.
 
         The location of the cursor is determined by emitting the ``u7`` terminal capability, or
-        VT100 `Query Cursor Position <http://www.termsys.demon.co.uk/vtansi.htm#status>`_ when such
-        capability is undefined, which elicits a response from a reply string described by
+        VT100 `Query Cursor Position
+        <https://www2.ccs.neu.edu/research/gpc/VonaUtils/vona/terminal/vtansi.htm#status>`_
+        when such capability is undefined, which elicits a response from a reply string described by
         capability ``u6``, or again VT100's definition of ``\x1b[%i%d;%dR`` when undefined.
 
         The ``(y, x)`` return value matches the parameter order of the :meth:`move_xy` capability.
@@ -1030,6 +1032,22 @@ class Terminal(object):
             width = self.width
         return Sequence(text, self).center(width, fillchar)
 
+    def truncate(self, text, width=None):
+        r"""
+        Truncate ``text`` to maximum ``width`` printable characters, retaining terminal sequences.
+
+        :arg str text: Text to truncate
+        :arg int width: The maximum width to truncate it to
+        :rtype: str
+        :returns: ``text`` truncated to at most ``width`` printable characters
+
+        >>> term.truncate(u'xyz\x1b[0;3m', 2)
+        u'xy\x1b[0;3m'
+        """
+        if width is None:
+            width = self.width
+        return Sequence(text, self).truncate(width)
+
     def length(self, text):
         u"""
         Return printable length of a string containing sequences.
@@ -1106,20 +1124,33 @@ class Terminal(object):
         """
         return Sequence(text, self).strip_seqs()
 
-    def split_seqs(self, text, **kwds):
+    def split_seqs(self, text, maxsplit=0):
         r"""
         Return ``text`` split by individual character elements and sequences.
 
         :arg str text: String containing sequences
-        :arg kwds: remaining keyword arguments for :func:`re.split`.
+        :arg int maxsplit: When maxsplit is nonzero, at most maxsplit splits
+            occur, and the remainder of the string is returned as the final element
+            of the list (same meaning is argument for :func:`re.split`).
         :rtype: list[str]
         :returns: List of sequences and individual characters
 
         >>> term.split_seqs(term.underline(u'xyz'))
         ['\x1b[4m', 'x', 'y', 'z', '\x1b(B', '\x1b[m']
+
+        >>> term.split_seqs(term.underline(u'xyz'), 1)
+        ['\x1b[4m', r'xyz\x1b(B\x1b[m']
         """
         pattern = self._caps_unnamed_any
-        return list(filter(None, re.split(pattern, text, **kwds)))
+        result = []
+        for idx, match in enumerate(re.finditer(pattern, text)):
+            result.append(match.group())
+            if maxsplit and idx == maxsplit:
+                remaining = text[match.end():]
+                if remaining:
+                    result[-1] += remaining
+                break
+        return result
 
     def wrap(self, text, width=None, **kwargs):
         """
@@ -1142,9 +1173,7 @@ class Terminal(object):
         wrapper = SequenceTextWrapper(width=width, term=self, **kwargs)
         lines = []
         for line in text.splitlines():
-            lines.extend(
-                (_linewrap for _linewrap in wrapper.wrap(line))
-                if line.strip() else (u'',))
+            lines.extend(iter(wrapper.wrap(line)) if line.strip() else (u'',))
 
         return lines
 
@@ -1198,7 +1227,7 @@ class Terminal(object):
         ready_r = [None, ]
         check_r = [self._keyboard_fd] if self._keyboard_fd is not None else []
 
-        while HAS_TTY and True:
+        while HAS_TTY:
             try:
                 ready_r, _, _ = select.select(check_r, [], [], timeout)
             except InterruptedError:
@@ -1360,6 +1389,13 @@ class Terminal(object):
         .. note:: When used without the context manager :meth:`cbreak`, or
             :meth:`raw`, :obj:`sys.__stdin__` remains line-buffered, and this
             function will block until the return key is pressed!
+
+        .. note:: On Windows, a 10 ms sleep is added to the key press detection loop to reduce CPU
+            load. Due to the behavior of :py:func:`time.sleep` on Windows, this will actually
+            result in a 15.6 ms delay when using the default `time resolution
+            <https://docs.microsoft.com/en-us/windows/win32/api/timeapi/nf-timeapi-timebeginperiod>`_.
+            Decreasing the time resolution will reduce this to 10 ms, while increasing it, which
+            is rarely done, will have a perceptable impact on the behavior.
         """
         resolve = functools.partial(resolve_sequence,
                                     mapper=self._keymap,
