@@ -9,7 +9,7 @@ import msvcrt  # pylint: disable=import-error
 import contextlib
 
 # 3rd party
-import jinxed.win32 as win32  # pylint: disable=import-error
+from jinxed import win32  # pylint: disable=import-error
 
 # local
 from .terminal import WINSZ
@@ -69,6 +69,7 @@ class Terminal(_Terminal):
             if timeout is not None and end < time.time():
                 break
 
+            time.sleep(0.01)  # Sleep to reduce CPU load
         return False
 
     @staticmethod
