@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+"""Tests for Terminal methods that account for sequences in strings"""
+
 # std imports
 import os
 import sys
@@ -10,8 +12,9 @@ import itertools
 import six
 import pytest
 
-from .accessories import (  # isort:skip
-    TestTerminal, as_subprocess, all_terms, many_lines, many_columns)
+# local
+from .accessories import TestTerminal, as_subprocess
+from .conftest import IS_WINDOWS
 
 if platform.system() != 'Windows':
     import fcntl
@@ -19,6 +22,7 @@ if platform.system() != 'Windows':
 
 
 def test_length_cjk():
+    """Test length of East Asian characters"""
     @as_subprocess
     def child():
         term = TestTerminal()
@@ -34,6 +38,7 @@ def test_length_cjk():
 
 
 def test_length_ansiart():
+    """Test length of ANSI art"""
     @as_subprocess
     def child(kind):
         import codecs
@@ -41,7 +46,8 @@ def test_length_ansiart():
         # this 'ansi' art contributed by xzip!impure for another project,
         # unlike most CP-437 DOS ansi art, this is actually utf-8 encoded.
         fname = os.path.join(os.path.dirname(__file__), 'wall.ans')
-        lines = codecs.open(fname, 'r', 'utf-8').readlines()
+        with codecs.open(fname, 'r', 'utf-8') as ansiart:
+            lines = ansiart.readlines()
         assert term.length(lines[0]) == 67  # ^[[64C^[[34m▄▓▄
         assert term.length(lines[1]) == 75
         assert term.length(lines[2]) == 78
@@ -49,14 +55,13 @@ def test_length_ansiart():
         assert term.length(lines[4]) == 78
         assert term.length(lines[5]) == 78
         assert term.length(lines[6]) == 77
-    kind = 'xterm-256color'
-    if platform.system() == 'Windows':
-        kind = 'vtwin10'
+    kind = 'vtwin10' if IS_WINDOWS else 'xterm-256color'
     child(kind)
 
 
 def test_sequence_length(all_terms):
     """Ensure T.length(string containing sequence) is correcterm."""
+    # pylint: disable=too-complex,too-many-statements
     @as_subprocess
     def child(kind):
         term = TestTerminal(kind=kind, force_styling=True)
@@ -321,10 +326,11 @@ def test_env_winsize():
     child()
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires fcntl")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires fcntl")
 def test_winsize(many_lines, many_columns):
     """Test height and width is appropriately queried in a pty."""
     pixel_width, pixel_height = 1024, 768
+
     @as_subprocess
     def child(lines=25, cols=80):
         # set the pty's virtual window size
@@ -343,14 +349,14 @@ def test_winsize(many_lines, many_columns):
 
 
 def test_Sequence_alignment_fixed_width(all_terms):
+    """Test alignment methods with width provided"""
     @as_subprocess
     def child(kind):
         term = TestTerminal(kind=kind)
         pony_msg = 'pony express, all aboard, choo, choo!'
         pony_len = len(pony_msg)
-        pony_colored = u''.join(
-            ['%s%s' % (term.color(n % 7), ch,)
-             for n, ch in enumerate(pony_msg)])
+        pony_colored = u''.join('%s%s' % (term.color(n % 7), ch,)
+                                for n, ch in enumerate(pony_msg))
         pony_colored += term.normal
         ladjusted = term.ljust(pony_colored, 88)
         radjusted = term.rjust(pony_colored, 88)
@@ -366,7 +372,7 @@ def test_Sequence_alignment_fixed_width(all_terms):
     child(kind=all_terms)
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires fcntl")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires fcntl")
 def test_Sequence_alignment(all_terms):
     """Tests methods related to Sequence class, namely ljust, rjust, center."""
     @as_subprocess
@@ -378,9 +384,8 @@ def test_Sequence_alignment(all_terms):
 
         pony_msg = 'pony express, all aboard, choo, choo!'
         pony_len = len(pony_msg)
-        pony_colored = u''.join(
-            ['%s%s' % (term.color(n % 7), ch,)
-             for n, ch in enumerate(pony_msg)])
+        pony_colored = u''.join('%s%s' % (term.color(n % 7), ch,)
+                                for n, ch in enumerate(pony_msg))
         pony_colored += term.normal
         ladjusted = term.ljust(pony_colored)
         radjusted = term.rjust(pony_colored)
@@ -461,12 +466,12 @@ def test_sequence_is_movement_false(all_terms):
     def child(kind):
         from blessed.sequences import measure_length
         term = TestTerminal(kind=kind)
-        assert (0 == measure_length(u'', term))
+        assert measure_length(u'', term) == 0
         # not even a mbs
-        assert (0 == measure_length(u'xyzzy', term))
+        assert measure_length(u'xyzzy', term) == 0
         # negative numbers, though printable as %d, do not result
         # in movement; just garbage. Also not a valid sequence.
-        assert (0 == measure_length(term.cuf(-333), term))
+        assert measure_length(term.cuf(-333), term) == 0
         assert (len(term.clear_eol) == measure_length(term.clear_eol, term))
         # various erases don't *move*
         assert (len(term.clear_bol) == measure_length(term.clear_bol, term))
@@ -493,10 +498,10 @@ def test_sequence_is_movement_false(all_terms):
     child(all_terms)
 
 
-def test_termcap_will_move_false(all_terms):
+def test_termcap_will_move_false(all_terms):  # pylint: disable=too-complex,too-many-branches
     """Test parser about sequences that do not move the cursor."""
     @as_subprocess
-    def child(kind):
+    def child(kind):  # pylint: disable=too-many-branches
         from blessed.sequences import iter_parse
         term = TestTerminal(kind=kind)
         if term.clear_eol:
