@@ -1,5 +1,5 @@
-# -*- coding: utf-8 -*-
 """Tests for Terminal() sequences and sequence-awareness."""
+# -*- coding: utf-8 -*-
 # std imports
 import sys
 import platform
@@ -9,10 +9,16 @@ import six
 import pytest
 
 # local
-from .accessories import TestTerminal, all_terms, unicode_cap, unicode_parm, as_subprocess
+from .accessories import TestTerminal, unicode_cap, unicode_parm, as_subprocess, MockTigetstr
+from .conftest import IS_WINDOWS
+
+try:
+    from unittest import mock
+except ImportError:
+    import mock
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires real tty")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires real tty")
 def test_capability():
     """Check that capability lookup works."""
     @as_subprocess
@@ -192,15 +198,16 @@ def test_vertical_location(all_terms):
     child(all_terms)
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires multiprocess")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires multiprocess")
 def test_inject_move_x():
     """Test injection of hpa attribute for screen/ansi (issue #55)."""
     @as_subprocess
     def child(kind):
         t = TestTerminal(kind=kind, stream=six.StringIO(), force_styling=True)
         COL = 5
-        with t.location(x=COL):
-            pass
+        with mock.patch('curses.tigetstr', side_effect=MockTigetstr(hpa=None)):
+            with t.location(x=COL):
+                pass
         expected_output = u''.join(
             (unicode_cap('sc') or u'\x1b[s',
              u'\x1b[{0}G'.format(COL + 1),
@@ -213,15 +220,16 @@ def test_inject_move_x():
     child('ansi')
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires multiprocess")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires multiprocess")
 def test_inject_move_y():
     """Test injection of vpa attribute for screen/ansi (issue #55)."""
     @as_subprocess
     def child(kind):
         t = TestTerminal(kind=kind, stream=six.StringIO(), force_styling=True)
         ROW = 5
-        with t.location(y=ROW):
-            pass
+        with mock.patch('curses.tigetstr', side_effect=MockTigetstr(vpa=None)):
+            with t.location(y=ROW):
+                pass
         expected_output = u''.join(
             (unicode_cap('sc') or u'\x1b[s',
              u'\x1b[{0}d'.format(ROW + 1),
@@ -234,9 +242,9 @@ def test_inject_move_y():
     child('ansi')
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires multiprocess")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires multiprocess")
 def test_inject_civis_and_cnorm_for_ansi():
-    """Test injection of cvis attribute for ansi."""
+    """Test injection of civis attribute for ansi."""
     @as_subprocess
     def child(kind):
         t = TestTerminal(kind=kind, stream=six.StringIO(), force_styling=True)
@@ -248,7 +256,7 @@ def test_inject_civis_and_cnorm_for_ansi():
     child('ansi')
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires multiprocess")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires multiprocess")
 def test_inject_sc_and_rc_for_ansi():
     """Test injection of sc and rc (save and restore cursor) for ansi."""
     @as_subprocess
@@ -280,6 +288,8 @@ def test_zero_location(all_terms):
 
 def test_mnemonic_colors(all_terms):
     """Make sure color shortcuts work."""
+    # pylint:  disable=consider-using-ternary
+
     @as_subprocess
     def child(kind):
         def color(t, num):
@@ -363,16 +373,10 @@ def test_formatting_functions(all_terms):
     def child(kind):
         t = TestTerminal(kind=kind)
         # test simple sugar,
-        if t.bold:
-            expected_output = u''.join((t.bold, u'hi', t.normal))
-        else:
-            expected_output = u'hi'
+        expected_output = u''.join((t.bold, u'hi', t.normal)) if t.bold else u'hi'
         assert t.bold(u'hi') == expected_output
         # Plain strs for Python 2.x
-        if t.green:
-            expected_output = u''.join((t.green, 'hi', t.normal))
-        else:
-            expected_output = u'hi'
+        expected_output = u''.join((t.green, 'hi', t.normal)) if t.green else u'hi'
         assert t.green('hi') == expected_output
         # Test unicode
         if t.underline:
@@ -470,31 +474,27 @@ def test_nice_formatting_errors(all_terms):
         t = TestTerminal(kind=kind)
         try:
             t.bold_misspelled('hey')
-            assert not t.is_a_tty or False, 'Should have thrown exception'
-        except TypeError:
-            e = sys.exc_info()[1]
+            assert not t.is_a_tty, 'Should have thrown exception'
+        except TypeError as e:
             assert 'Unknown terminal capability,' in e.args[0]
         try:
             t.bold_misspelled(u'hey')  # unicode
-            assert not t.is_a_tty or False, 'Should have thrown exception'
-        except TypeError:
-            e = sys.exc_info()[1]
+            assert not t.is_a_tty, 'Should have thrown exception'
+        except TypeError as e:
             assert 'Unknown terminal capability,' in e.args[0]
 
         try:
             t.bold_misspelled(None)  # an arbitrary non-string
-            assert not t.is_a_tty or False, 'Should have thrown exception'
-        except TypeError:
-            e = sys.exc_info()[1]
+            assert not t.is_a_tty, 'Should have thrown exception'
+        except TypeError as e:
             assert 'Unknown terminal capability,' not in e.args[0]
 
         if platform.python_implementation() != 'PyPy':
             # PyPy fails to toss an exception, Why?!
             try:
                 t.bold_misspelled('a', 'b')  # >1 string arg
-                assert not t.is_a_tty or False, 'Should have thrown exception'
-            except TypeError:
-                e = sys.exc_info()[1]
+                assert not t.is_a_tty, 'Should have thrown exception'
+            except TypeError as e:
                 assert 'Unknown terminal capability,' in e.args[0], e.args
 
     child(all_terms)
@@ -527,10 +527,7 @@ def test_padd():
         assert Sequence('xxxx\x1b[3Dzz', term).padd() == u'xzz'
         assert Sequence('\x1b[3D', term).padd() == u''  # "Trim left"
         assert Sequence(term.red('xxxx\x1b[3Dzz'), term).padd() == term.red(u'xzz')
-
-    kind = 'xterm-256color'
-    if platform.system() == 'Windows':
-        kind = 'vtwin10'
+    kind = 'vtwin10' if IS_WINDOWS else 'xterm-256color'
     child(kind)
 
 
@@ -547,9 +544,57 @@ def test_split_seqs(all_terms):
             result = list(term.split_seqs(given_text))
             assert result == expected
 
+    child(all_terms)
+
+
+def test_split_seqs_maxsplit1(all_terms):
+    """Test Terminal.split_seqs with maxsplit=1."""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+
         if term.bold:
             given_text = term.bold + 'bbq'
-            expected = [term.bold, 'b', 'b', 'q']
+            expected = [term.bold, 'bbq']
+            result = list(term.split_seqs(given_text, 1))
+            assert result == expected
+
+    child(all_terms)
+
+
+def test_split_seqs_term_right(all_terms):
+    """Test Terminal.split_seqs with parameterized sequence"""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+
+        if term.move_up:
+            given_text = 'XY' + term.move_right + 'VK'
+            expected = ['X', 'Y', term.move_right, 'V', 'K']
+            result = list(term.split_seqs(given_text))
+            assert result == expected
+
+    child(all_terms)
+
+
+def test_split_seqs_maxsplit3_and_term_right(all_terms):
+    """Test Terminal.split_seqs with parameterized sequence."""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+
+        if term.move_right(32):
+            given_text = 'PQ' + term.move_right(32) + 'RS'
+            expected = ['P', 'Q', term.move_right(32), 'RS']
+            result = list(term.split_seqs(given_text, 3))
+            assert result == expected
+
+        if term.move_up(45):
+            given_text = 'XY' + term.move_up(45) + 'VK'
+            expected = ['X', 'Y', term.move_up(45), 'V', 'K']
             result = list(term.split_seqs(given_text))
             assert result == expected
 
@@ -584,6 +629,7 @@ def test_formatting_other_string(all_terms):
 def test_termcap_match_optional():
     """When match_optional is given, numeric matches are optional"""
     from blessed.sequences import Termcap
+
     @as_subprocess
     def child():
         t = TestTerminal(force_styling=True)
@@ -603,3 +649,82 @@ def test_termcap_match_optional():
         assert cap.re_compiled.match(t.cub1) is None
 
     child()
+
+
+def test_truncate(all_terms):
+    """Test terminal.truncate and make sure it agrees with terminal.length"""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+
+        test_string = term.red("Testing ") + term.yellow("makes ") +\
+            term.green("me ") + term.blue("feel ") +\
+            term.indigo("good") + term.normal
+        stripped_string = term.strip_seqs(test_string)
+        for i in range(len(stripped_string)):
+            test_l = term.length(term.truncate(test_string, i))
+            assert test_l == len(stripped_string[:i])
+        test_nogood = term.red("Testing ") + term.yellow("makes ") +\
+            term.green("me ") + term.blue("feel ") +\
+            term.indigo("") + term.normal
+        trunc = term.truncate(test_string, term.length(test_string) - len("good"))
+        assert trunc == test_nogood
+
+    child(all_terms)
+
+
+def test_truncate_wide_end(all_terms):
+    """Ensure that terminal.truncate has the correct behaviour for wide characters."""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+        test_string = u"AB\uff23"  # ABＣ
+        assert term.truncate(test_string, 3) == u"AB"
+
+    child(all_terms)
+
+
+def test_truncate_wcwidth_clipping(all_terms):
+    """Ensure that terminal.truncate has the correct behaviour for wide characters."""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+        assert term.truncate("", 4) == ""
+        test_string = term.blue(u"one\x01two")
+        assert term.truncate(test_string, 4) == term.blue(u"one\x01t")
+
+    child(all_terms)
+
+
+def test_truncate_padding(all_terms):
+    """Ensure that terminal.truncate has the correct behaviour for wide characters."""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+        test_right_string = term.blue(u"one" + term.move_right(5) + u"two")
+        assert term.truncate(test_right_string, 9) == term.blue(u"one     t")
+
+        test_bs_string = term.blue(u"one\b\b\btwo")
+        assert term.truncate(test_bs_string, 3) == term.blue(u"two")
+
+    if all_terms != 'vtwin10':
+        # padding doesn't work the same on windows !
+        child(all_terms)
+
+
+def test_truncate_default(all_terms):
+    """Ensure that terminal.truncate functions with the default argument."""
+    @as_subprocess
+    def child(kind):
+        from blessed import Terminal
+        term = Terminal(kind)
+        test = "Testing " + term.red("attention ") + term.blue("please.")
+        trunc = term.truncate(test)
+        assert term.length(trunc) <= term.width
+        assert term.truncate(term.red('x' * 1000)) == term.red('x' * term.width)
+
+    child(all_terms)
